/***
 * FractalGUI: a graphical tool to edit Fractal component configurations.
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Eric Bruneton, Patrice Fauvel
 */

package org.objectweb.fractal.gui;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;

import org.objectweb.fractal.adl.BasicParser;
import org.objectweb.fractal.adl.Loader;
import org.objectweb.fractal.adl.ParserException;
import org.objectweb.fractal.adl.ResolvingLoader;
import org.objectweb.fractal.adl.XMLElement;

import org.objectweb.fractal.util.Fractal;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.Map;

/**
 * FractalGUI launcher.
 */

public class FractalGUI {

  /**
   * Launches FractalGUI.
   *
   * @param args the application's arguments.
   */

  public static void main (String[] args) {
    try {
      if (System.getProperty("fractal.provider") == null) {
        System.setProperty(
          "fractal.provider", "org.objectweb.fractal.julia.Julia");
        System.setProperty(
          "julia.loader", "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty(
          "julia.config", "org/objectweb/fractal/gui/julia.cfg");
      }

      String templateFile = "org/objectweb/fractal/gui/fractalgui.fgl";
      if (args.length > 0) {
        templateFile = args[0];
      }

      // manual assembly of the parser components
      Component boot = Fractal.getBootstrapComponent();
      FileLoader bl = new FileLoader(templateFile);
      ResolvingLoader rl = new ResolvingLoader();
      rl.bindFc("delegate", bl);
      BasicParser bp = new BasicParser();
      bp.bindFc("loader", rl);
      bp.bindFc("type-factory", Fractal.getTypeFactory(boot));
      bp.bindFc("generic-factory", Fractal.getGenericFactory(boot));

      // creates and instantiates the application's template
      System.out.println("Creating components from ADL definition...");
      Component comp = bp.loadTemplate("org.objectweb.fractal.gui.FractalGUI", false);

      // starts the application if it has a lifecycle controller
      System.out.println("Starting application...");
      LifeCycleController lc = null;
      try {
        lc = Fractal.getLifeCycleController(comp);
      } catch (NoSuchInterfaceException ignored) {
      }
      if (lc != null) {
        lc.startFc();
      }
      System.out.println("Ready.");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * A Fractal ADL loader that loads definitions from a single file.
   */

  static class FileLoader implements Loader {

    private Map definitions;

    public FileLoader (final String file) throws IOException, ParserException {
      definitions = new HashMap();
      InputStream is;
      try {
        is = new FileInputStream(file);
      } catch (IOException ioe) {
        is = getClass().getResourceAsStream("/" + file);
        if (is == null) {
          throw new IOException("Cannot find file '" + file + "'");
        }
      }
      LineNumberReader r = new LineNumberReader(new InputStreamReader(is));
      while (true) {
        XMLElement e = XMLElement.parse(r, false);
        if (e == null) {
          return;
        } else {
          e.setAttribute("[source]", file + ":" + Integer.toString(e.line));
        }
        if (!e.tag.equals("component-type") &&
            !e.tag.equals("primitive-template") &&
            !e.tag.equals("composite-template") &&
            !e.tag.equals("external-template"))
        {
          continue;
        }
        String name = e.getAttribute("name", null);
        definitions.put(name, e);
      }
    }

    // -------------------------------------------------------------------------
    // Implementation of the Loader interface
    // -------------------------------------------------------------------------

    public XMLElement loadTypeDescriptor (final String name)
      throws ParserException
    {
      XMLElement result = (XMLElement)definitions.get(name);
      if (result == null) {
        throw new ParserException("'" + name + "' not found", null);
      }
      return result;
    }

    public XMLElement loadTemplateDescriptor (final String name)
      throws ParserException
    {
      XMLElement result = (XMLElement)definitions.get(name);
      if (result == null) {
        throw new ParserException("'" + name + " 'not found", null);
      }
      return result;
    }
  }
}
